// Mesh related utility functions - Written by Dr. Ryan Durscher AFRL/RQVC

#include "meshTypes.h"  // Bring in mesh structures
#include "capsTypes.h"  // Bring in CAPS types
#include "cfdTypes.h"   // Bring in cfd structures
#include "feaTypes.h"   // Bring in fea structures
#include "miscTypes.h"  // Bring in miscellanous structures

#ifdef __cplusplus
extern "C" {
#endif

int mesh_bodyTessellation(ego tess,
						  mapAttrToIndexStruct *attrMap,
						  int *numNodes,
						  double *xyzCoord[],
         	 	 	 	  int *numTriFace, int *triFaceConn[], int *triFaceCompID[],
						  int *numBndEdge, int *bndEdgeConn[], int *bndEdgeCompID[],
						  int *tessFaceQuadMap,
						  int *numQuadFace, int *quadFaceConn[], int *quadFaceCompID[]);

int write_MAPBC(char *fname,
                 int numBnds,
                 int *bndIds,
                 int *bndVals);

int write_STL(char   *fname,
              int    numTris,
              double *verts,
              int    *conn);

void get_Surface_Norm(double p1[3],
                      double p2[3],
                      double p3[3],
                      double norm[3]);

// Populate bndCondStruct boundary condition information - Boundary condition values get filled with 99
int populate_bndCondStruct_from_bcPropsStruct(cfdBCsStruct *bcProps,
                                              bndCondStruct *bndConds);

// Populate bndCondStruct boundary condition information from attribute map - Boundary condition values get filled with 99
int populate_bndCondStruct_from_mapAttrToIndexStruct(mapAttrToIndexStruct *attrMap,
                                                     bndCondStruct *bndConds);

// Initiate (0 out all values and NULL all pointers) a bndCond in the bndCondStruct structure format
int initiate_bndCondStruct(bndCondStruct *bndCond);

// Destroy (0 out all values, free all arrays, and NULL all pointers) a bndCond in the bndCondStruct structure format
int destroy_bndCondStruct(bndCondStruct *bndCond);

// Initiate (0 out all values and NULL all pointers) a meshInput in the meshInputStruct structure format
int initiate_meshInputStruct(meshInputStruct *meshInput);

// Destroy (0 out all values and NULL all pointers) a meshInput in the meshInputStruct structure format
int destroy_meshInputStruct(meshInputStruct *meshInput);

// Initiate (0 out all values and NULL all pointers) a bodyTessMapping in the bodyTessMappingStruct structure format
int initiate_bodyTessMappingStruct (bodyTessMappingStruct *bodyTessMapping);

// Destroy (0 out all values and NULL all pointers) a bodyTessMapping in the bodyTessMappingStruct structure format
int destroy_bodyTessMappingStruct (bodyTessMappingStruct *bodyTessMapping);

// Initiate (0 out all values and NULL all pointers) a meshProp in the meshSizingStruct structure format
int initiate_meshSizingStruct (meshSizingStruct *meshProp);

// Destroy (0 out all values and NULL all pointers) a meshProp in the meshSizingStruct structure format
int destroy_meshSizingStruct (meshSizingStruct *meshProp);

// Fill meshProps in a meshBCStruct format with mesh boundary condition information from incoming Mesh Sizing Tuple
int mesh_getSizingProp(int numTuple,
                   	   capsTuple meshBCTuple[],
					   mapAttrToIndexStruct *attrMap,
					   int *numMeshProp,
					   meshSizingStruct *meshProps[]);

// Modify the EGADS body tessellation based on given inputs
int mesh_modifyBodyTess(int numMeshProp,
						meshSizingStruct meshProp[],
						int minEdgePointGlobal,
						int maxEdgePointGlobal,
						double tessParamGlobal[],
						mapAttrToIndexStruct attrMap,
						int numBody,
						ego *bodies[]);

// Initiate (0 out all values and NULL all pointers) a CFD mesh data in the cfdMeshDataStruct structure format
int initiate_cfdMeshDataStruct (cfdMeshDataStruct *data);

// Destroy (0 out all values and NULL all pointers) a CFD mesh data in the cfdMeshDataStruct structure format
int destroy_cfdMeshDataStruct (cfdMeshDataStruct *data);

// Copy a CFD mesh data in the cfdMeshDataStruct structure format
int copy_cfdMeshDataStruct (cfdMeshDataStruct *dataIn, cfdMeshDataStruct *dataOut);

// Initiate (0 out all values and NULL all pointers) a FEA mesh data in the feaMeshDataStruct structure format
int initiate_feaMeshDataStruct (feaMeshDataStruct *data);

// Destroy (0 out all values and NULL all pointers) a FEA mesh data in the feaMeshDataStruct structure format
int destroy_feaMeshDataStruct (feaMeshDataStruct *data);

// Copy a FEA mesh data in the feaMeshDataStruct structure format
int copy_feaMeshDataStruct (feaMeshDataStruct *dataIn, feaMeshDataStruct *dataOut);

// Initiate  (0 out all values and NULL all pointers) and allocate the analysisData void pointer. Creation selected based on type.
int initiate_analysisData(void **analysisData, meshAnalysisTypeEnum analysisType);

// Destroy (0 out all values and NULL all pointers) and free of the analysisData void pointer. Correct destroy function selected based on type.
int destroy_analysisData(void **analysisData, meshAnalysisTypeEnum analysisType);

// Initiate (0 out all values and NULL all pointers) a node data in the meshNode structure format
int initiate_meshNodeStruct(meshNodeStruct *node, meshAnalysisTypeEnum meshAnalysisType);

// Destroy (0 out all values and NULL all pointers) a node data in the meshNode structure format
int destroy_meshNodeStruct(meshNodeStruct *node);

// Initiate (0 out all values and NULL all pointers) a element data in the meshElement structure format
int initiate_meshElementStruct(meshElementStruct *element, meshAnalysisTypeEnum meshAnalysisType);

// Destroy (0 out all values and NULL all pointers) a element data in the meshElementStruct structure format
int destroy_meshElementStruct(meshElementStruct *element);

// Initiate (0 out all values and NULL all pointers) the meshQuickRef data in the meshQuickRefStruct structure format
int initiate_meshQuickRefStruct(meshQuickRefStruct *quickRef);

// Destroy (0 out all values and NULL all pointers) the meshQuickRef data in the meshQuickRefStruct structure format
int destroy_meshQuickRefStruct(meshQuickRefStruct *quickRef);

// Destroy (0 out all values and NULL all pointers) all nodes in the mesh
int destroy_meshNodes(meshStruct *mesh);

// Destroy (0 out all values and NULL all pointers) all elements in the mesh
int destroy_meshElements(meshStruct *mesh);

// Initiate (0 out all values and NULL all pointers) a mesh data in the meshStruct structure format
int initiate_meshStruct(meshStruct *mesh);

// Destroy (0 out all values and NULL all pointers) a mesh data in the meshStruct structure format
int destroy_meshStruct(meshStruct *mesh);

// Create a surface mesh in meshStruct format using the EGADS body object
int mesh_surfaceMeshEGADSBody(ego body, double tessParams[3], mapAttrToIndexStruct *attrMap, meshStruct *surfMesh);

// Create a surface mesh in meshStruct format using the EGADS body tessellation
int mesh_surfaceMeshEGADSTess(ego egadsTess, mapAttrToIndexStruct *attrMap, meshStruct *surfMesh);

// Return the number of connectivity points based on type
int mesh_numMeshConnectivity(meshElementTypeEnum elementType);

// Return the number of connectivity points based on type of element provided
int mesh_numMeshElementConnectivity(meshElementStruct *element);

// Allocate mesh element connectivity array based on type
int mesh_allocMeshElementConnectivity(meshElementStruct *element);

// Retrieve the number of mesh element of a given type
int mesh_retrieveNumMeshElements(int numElement,
							  meshElementStruct element[],
							  meshElementTypeEnum elementType,
							  int *numElementType);

// Retrieve the starting index of a given type -assume elements were put in order
int mesh_retrieveStartIndexMeshElements(int numElement,
										meshElementStruct element[],
										meshElementTypeEnum elementType,
										int *numElementType,
										int *startIndex);

// Retrieve list of mesh element of a given type - elementTypeList is freeable
int mesh_retrieveMeshElements(int numElement,
							  meshElementStruct element[],
							  meshElementTypeEnum elementType,
							  int *numElementType,
							  int *elementTypeList[]);

// Fill out the QuickRef lists for all element types
int mesh_fillQuickRefList( meshStruct *mesh);

// Make a copy of the analysis Data
int mesh_copyMeshAnalysisData(void *in, meshAnalysisTypeEnum analysisType, void *out);

// Make a copy of an element - may offset the element and connectivity indexing
int mesh_copyMeshElementStruct(meshElementStruct *in, int elementOffSetIndex, int connOffSetIndex, meshElementStruct *out);

// Make a copy of an node - may offset the node indexing
int mesh_copyMeshNodeStruct(meshNodeStruct *in, int nodeOffSetIndex, meshNodeStruct *out);

// Combine mesh structures
int mesh_combineMeshStruct(int numMesh, meshStruct mesh[], meshStruct *combineMesh );

// Write a mesh contained in the mesh structure in AFLR3 format (*.ugrid, *.lb8.ugrid, *.b8.ugrid)
int mesh_writeAFLR3(char *fname,
					int asciiFlag, // 0 for binary, anything else for ascii
					meshStruct *mesh,
					double scaleFactor); // Scale factor for coordinates

// Write a mesh contained in the mesh structure in VTK format (*.vtk)
int mesh_writeVTK(char *fname,
				  int asciiFlag, // 0 for binary, anything else for ascii
				  meshStruct *mesh,
				  double scaleFactor); // Scale factor for coordinates

// Write a mesh contained in the mesh structure in SU2 format (*.su2)
int mesh_writeSU2(char *fname,
              	  int asciiFlag, // 0 for binary, anything else for ascii
				  meshStruct *mesh,
				  int numBnds,
				  int bndID[],
				  double scaleFactor); // Scale factor for coordinates

// Write a mesh contained in the mesh structure in NASTRAN format (*.bdf)
int mesh_writeNASTRAN(char *fname,
                  	  int asciiFlag, // 0 for binary, anything else for ascii
					  meshStruct *nasMesh,
					  feaFileTypeEnum gridFileType,
					  double scaleFactor); // Scale factor for coordinates

// Write a mesh contained in the mesh structure in Astros format (*.bdf)
int mesh_writeAstros(char *fname,
                     int asciiFlag, // 0 for binary, anything else for ascii
                     meshStruct *mesh,
                     feaFileTypeEnum gridFileType,
                     int numDesignVariable, feaDesignVariableStruct feaDesignVariable[],
					 double scaleFactor); // Scale factor for coordinates

// Write a mesh contained in the mesh structure in STL format (*.stl)
int mesh_writeSTL(char *fname,
				  int asciiFlag, // 0 for binary, anything else for ascii
				  meshStruct *mesh,
				  double scaleFactor); // Scale factor for coordinates

// Write a mesh contained in the mesh structure in Tecplot format (*.dat)
int mesh_writeTecplot(char *fname,
                      int asciiFlag,
                      meshStruct *mesh,
					  double scaleFactor); // Scale factor for coordinates

// Write a mesh contained in the mesh structure in Airfoil format (boundary edges only [Lines]) (*.af)
//  "Character Name"
//	x[0] y[0] x y coordinates
//	x[1] y[1]
//	 ...  ...
int mesh_writeAirfoil(char *fname,
                      int asciiFlag,
                      meshStruct *mesh,
					  double scaleFactor); // Scale factor for coordinates

// Write a mesh contained in the mesh structure in FAST mesh format (*.msh)
int mesh_writeFAST(char *fname,
        		   int asciiFlag,
				   meshStruct *mesh,
				   double scaleFactor); // Scale factor for coordinates

// Extrude a surface mesh a single unit the length of extrusionLength - return a
// volume mesh, cell volume and left-handness is not checked
int extrude_SurfaceMesh(double extrusionLength,
                        int extrusionMarker,
                        meshStruct *surfaceMesh, meshStruct *volumeMesh);
//////// EXPERIMENTAL ///////////////

#ifdef __cplusplus
}
#endif
