## [importPrint]
from __future__ import print_function
## [importPrint]

# Import pyCAPS and os module
## [import]
try:
    import pyCAPS
except:
    print ("Unable to import pyCAPS module")
    raise SystemError
try:
    import os
except:
    print ("Unable to import os module")
    raise SystemError
## [import]

# -----------------------------------------------------------------
# Initialize capsProblem object
# -----------------------------------------------------------------
## [initateProblem]
myProblem = pyCAPS.capsProblem()
## [initateProblem]

# -----------------------------------------------------------------
# Load CSM file and Change a design parameter - area in the geometry 
# Any despmtr from the avlWing.csm file are available inside the pyCAPS script
# They are: thick, camber, area, aspect, taper, sweep, washout, dihedral
# -----------------------------------------------------------------

## [geometry]
myGeometry = myProblem.loadCAPS("./csmData/avlWing.csm")

myGeometry.setGeometryVal("area", 10.0)
## [geometry]

# -----------------------------------------------------------------
# Load desired aim 
# -----------------------------------------------------------------
print ("Loading AIM")
## [loadAIM]
myAnalysis = myProblem.loadAIM(aim = "avlAIM", 
                               analysisDir = "AVLAnalysisTest",
                               capsIntent = "LINEARAERO")
## [loadAIM]
# -----------------------------------------------------------------
# Also available are all aimInput values
# Set new Mach/Alt parameters
# -----------------------------------------------------------------

## [setInputs]
myAnalysis.setAnalysisVal("Mach", 0.5)
myAnalysis.setAnalysisVal("Alpha", 1.0)
myAnalysis.setAnalysisVal("Beta", 0.0)

wing = {"groupName"  : "Wing", # Notice Wing is the value for the capsGroup attribute 
        "numChord"   : 8, 
        "spaceChord" : 1.0, 
        "numSpan"    : 12, 
        "spaceSpan"  : 1.0}

myAnalysis.setAnalysisVal("AVL_Surface", [("Wing", wing)])

## [setInputs]

# -----------------------------------------------------------------
# Run AIM pre-analysis
# -----------------------------------------------------------------
## [preAnalysis]
myAnalysis.aimPreAnalysis()
## [preAnalysis]

# -----------------------------------------------------------------
# Run AVL
# -----------------------------------------------------------------
## [runAVL]
print ("Running AVL")
currentDirectory = os.getcwd() # Get our current working directory 
os.chdir(myAnalysis.analysisDir) # Move into test directory
os.system("avl caps < avlInput.txt > avlOutput.txt");
os.chdir(currentDirectory) # Move back to working directory 
## [runAVL]

# -----------------------------------------------------------------
# Run AIM post-analysis
# -----------------------------------------------------------------
## [postAnalysis]
myAnalysis.aimPostAnalysis()
## [postAnalysis]

# -----------------------------------------------------------------
# Get Output Data from AVL
# These calls access aimOutput data
# -----------------------------------------------------------------
## [output]
print ("Alpha  " + str(myAnalysis.getAnalysisOutVal("Alpha")))
print ("Beta   " + str(myAnalysis.getAnalysisOutVal("Beta")))
print ("Mach   " + str(myAnalysis.getAnalysisOutVal("Mach")))
print ("pb/2V  " + str(myAnalysis.getAnalysisOutVal("pb/2V")))
print ("qc/2V  " + str(myAnalysis.getAnalysisOutVal("qc/2V")))
print ("rb/2V  " + str(myAnalysis.getAnalysisOutVal("rb/2V")))
print ("p'b/2V " + str(myAnalysis.getAnalysisOutVal("p'b/2V")))
print ("r'b/2V " + str(myAnalysis.getAnalysisOutVal("r'b/2V")))
print ("CXtot  " + str(myAnalysis.getAnalysisOutVal("CXtot")))
print ("CYtot  " + str(myAnalysis.getAnalysisOutVal("CYtot")))
print ("CZtot  " + str(myAnalysis.getAnalysisOutVal("CZtot")))
print ("Cltot  " + str(myAnalysis.getAnalysisOutVal("Cltot")))
print ("Cmtot  " + str(myAnalysis.getAnalysisOutVal("Cmtot")))
print ("Cntot  " + str(myAnalysis.getAnalysisOutVal("Cntot")))
print ("Cl'tot " + str(myAnalysis.getAnalysisOutVal("Cl'tot")))
print ("Cn'tot " + str(myAnalysis.getAnalysisOutVal("Cn'tot")))
print ("CLtot  " + str(myAnalysis.getAnalysisOutVal("CLtot")))
print ("CDtot  " + str(myAnalysis.getAnalysisOutVal("CDtot")))
print ("CDvis  " + str(myAnalysis.getAnalysisOutVal("CDvis")))
print ("CLff   " + str(myAnalysis.getAnalysisOutVal("CLff")))
print ("CYff   " + str(myAnalysis.getAnalysisOutVal("CYff")))
print ("CDind  " + str(myAnalysis.getAnalysisOutVal("CDind")))
print ("CDff   " + str(myAnalysis.getAnalysisOutVal("CDff")))
print ("e      " + str(myAnalysis.getAnalysisOutVal("e")))
## [output]
print("")
print("")
valueDict = myAnalysis.getAnalysisOutVal()
for i in valueDict.keys():
    print(i + " = " + str(valueDict[i]))
# -----------------------------------------------------------------
# Close CAPS 
# -----------------------------------------------------------------
myProblem.closeCAPS()